function x = fminx(fminfcn,ax,bx,tol,print,param)
%FMIN	Minimum of a function of one variable.
%	FMIN(F,x1,x2) returns the value of x that minimimizes F(x) in
%	the interval x1 < x < x2.  The value of x is accurate to within
%	a relative error of 1e-3.  F is a string containing the name
%	of the function.
%	FMIN(F,x1,x2,tol) returns x to a relative error of tol.
%	FMIN(F,x1,x2,tol,1,param) displays a trace of the algorithm steps.
%  param incluye los parametros de F

%  References:
%	[1] "Computer Methods for Mathematical Computations", Forsythe,
%	Malcolm, and Moler, Prentice-Hall, 1976.

%	Copyright (c) 1988 by the MathWorks, Inc.
%	Revised 10-14-88 JNL
%	Original coding by Duane Hanselman, University of Maine

% initialization

        if nargin <= 4, print = 0; end
        if nargin <= 3, tol = (bx-ax)/1000; end
        num = 1;
        seps = sqrt(eps);
        c = 0.5*(3.0 - sqrt(5.0));
        a = ax; b = bx;
        v = a + c*(b-a);
        w = v; x = v;
        d = 0.0; e = 0.0;
        fx = feval(fminfcn,x,param);
        if print, clc, fmin_data = [1 x fx], end
        fv = fx; fw = fx;
        xm = 0.5*(a+b);
        tol1 = seps*abs(x) + tol/3.0;   tol2 = 2.0*tol1;

% main loop

while ( abs(x-xm) > (tol2 - 0.5*(b-a)) )

        num = num+1; gs = 1;

        % is parabolic fit possible
        if abs(e) > tol1    % yes, so fit parabola
                gs = 0;
                r = (x-w)*(fx-fv);
                q = (x-v)*(fx-fw);
                p = (x-v)*q-(x-w)*r;
                q = 2.0*(q-r);
                if q > 0.0,  p = -p; end
                q = abs(q);
                r = e;  e = d;

                % is parabola acceptable
                if ( (abs(p)<abs(0.5*q*r)) | (p>q*(a-x)) | (p<q*(b-x)) )

                        % yes, parabolic interpolation step
                        d = p/q;
                        u = x+d;
                        step = '   num        x        fx       parabolic';

                        % f must not be evaluated too close to ax or bx
                        if ((u-a) < tol2) | ((b-u) < tol2)
                                si = sign(xm-x) + ((xm-x) == 0);
                                d = tol1*si;
                        end
                else    % not acceptable, must do a golden section step
                        gs=1;
                end
        end
        if gs    % a golden-section step is required

                if x >= xm, e = a-x;    else, e = b-x;  end
                        d = c*e;
                        step = '   num        x        fx       golden   ';
        end

        % func must not be evaluated too close to x
        si = sign(d) + (d == 0);
        u = x + si * max( abs(d), tol1 );
        fu = feval(fminfcn,u,param);
        if print, clc, fmin_data = [num u fu], disp(step), end

        % update a, b, v, w, x, xm, tol1, tol2
        if fu <= fx
                if u >= x, a = x; else, b = x; end
                v = w; fv = fw;
                w = x; fw = fx;
                x = u; fx = fu;
        else % fu > fx
                if u < x, a = u; else,b = u; end
                if ( (fu <= fw) | (w == x) )
                        v = w; fv = fw;
                        w = u; fw = fu;
                elseif ( (fu <= fv) | (v == x) | (v == w) )
                        v = u; fv = fu;
                end
        end
	xm = 0.5*(a+b);
	tol1 = seps*abs(x) + tol/3.0; tol2 = 2.0*tol1;
end
